# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2021/10/21 12:19
Desc: 巨潮资讯-数据中心-专题统计-基金报表
http://webapi.cninfo.com.cn/#/thematicStatistics
"""
import time

import pandas as pd
import requests
from py_mini_racer import py_mini_racer

js_str = """
    function mcode(input) {  
                var keyStr = "ABCDEFGHIJKLMNOP" + "QRSTUVWXYZabcdef" + "ghijklmnopqrstuv"   + "wxyz0123456789+/" + "=";  
                var output = "";  
                var chr1, chr2, chr3 = "";  
                var enc1, enc2, enc3, enc4 = "";  
                var i = 0;  
                do {  
                    chr1 = input.charCodeAt(i++);  
                    chr2 = input.charCodeAt(i++);  
                    chr3 = input.charCodeAt(i++);  
                    enc1 = chr1 >> 2;  
                    enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);  
                    enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);  
                    enc4 = chr3 & 63;  
                    if (isNaN(chr2)) {  
                        enc3 = enc4 = 64;  
                    } else if (isNaN(chr3)) {  
                        enc4 = 64;  
                    }  
                    output = output + keyStr.charAt(enc1) + keyStr.charAt(enc2)  
                            + keyStr.charAt(enc3) + keyStr.charAt(enc4);  
                    chr1 = chr2 = chr3 = "";  
                    enc1 = enc2 = enc3 = enc4 = "";  
                } while (i < input.length);  
          
                return output;  
            }  
"""


def fund_report_stock_cninfo(date: str = "20210630") -> pd.DataFrame:
    """
    巨潮资讯-数据中心-专题统计-基金报表-基金重仓股
    http://webapi.cninfo.com.cn/#/thematicStatistics
    :param date: 报告时间; choice of {"XXXX0331", "XXXX0630", "XXXX0930", "XXXX1231"}
    :type date: str
    :return: 基金重仓股
    :rtype: pandas.DataFrame
    """
    url = "http://webapi.cninfo.com.cn/api/sysapi/p_sysapi1112"
    random_time_str = str(int(time.time()))
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(js_str)
    mcode = js_code.call("mcode", random_time_str)
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "mcode": mcode,
        "Origin": "http://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "http://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    params = {
        "rdate": "-".join([date[:4], date[4:6], date[6:]]),
    }
    r = requests.post(url, headers=headers, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.rename(
        columns={
            "F003N": "持股总市值",
            "F002N": "持股总数",
            "F001N": "基金覆盖家数",
            "SECNAME": "股票简称",
            "ID": "序号",
            "SECCODE": "股票代码",
            "ENDDATE": "报告期",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "序号",
            "股票代码",
            "股票简称",
            "报告期",
            "基金覆盖家数",
            "持股总数",
            "持股总市值",
        ]
    ]
    temp_df["报告期"] = pd.to_datetime(temp_df["报告期"]).dt.date
    temp_df["持股总数"] = pd.to_numeric(temp_df["持股总数"])
    temp_df["持股总市值"] = pd.to_numeric(temp_df["持股总市值"])
    temp_df["基金覆盖家数"] = pd.to_numeric(temp_df["基金覆盖家数"])
    return temp_df


def fund_report_industry_allocation_cninfo(date: str = "20210630") -> pd.DataFrame:
    """
    巨潮资讯-数据中心-专题统计-基金报表-基金行业配置
    http://webapi.cninfo.com.cn/#/thematicStatistics
    :param date: 报告时间; choice of {"XXXX0331", "XXXX0630", "XXXX0930", "XXXX1231"}, 从 2017 年开始
    :type date: str
    :return: 基金行业配置
    :rtype: pandas.DataFrame
    """
    url = "http://webapi.cninfo.com.cn/api/sysapi/p_sysapi1113"
    random_time_str = str(int(time.time()))
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(js_str)
    mcode = js_code.call("mcode", random_time_str)
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "mcode": mcode,
        "Origin": "http://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "http://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    params = {
        "rdate": "-".join([date[:4], date[4:6], date[6:]]),
    }
    r = requests.post(url, headers=headers, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.rename(
        columns={
            'F004N': '行业规模',
            'F003N': '基金覆盖家数',
            'F002V': '证监会行业名称',
            'F001V': '行业编码',
            'ENDDATE': '报告期',
            'F005N': '占净资产比例'
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            '行业编码',
            '证监会行业名称',
            '报告期',
            '基金覆盖家数',
            '行业规模',
            '占净资产比例',
        ]
    ]
    temp_df["报告期"] = pd.to_datetime(temp_df["报告期"]).dt.date
    temp_df["基金覆盖家数"] = pd.to_numeric(temp_df["基金覆盖家数"])
    temp_df["行业规模"] = pd.to_numeric(temp_df["行业规模"])
    temp_df["占净资产比例"] = pd.to_numeric(temp_df["占净资产比例"])
    return temp_df


def fund_report_asset_allocation_cninfo() -> pd.DataFrame:
    """
    巨潮资讯-数据中心-专题统计-基金报表-基金资产配置
    http://webapi.cninfo.com.cn/#/thematicStatistics
    :return: 基金资产配置
    :rtype: pandas.DataFrame
    """
    url = "http://webapi.cninfo.com.cn/api/sysapi/p_sysapi1114"
    random_time_str = str(int(time.time()))
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(js_str)
    mcode = js_code.call("mcode", random_time_str)
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "mcode": mcode,
        "Origin": "http://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "http://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    r = requests.post(url, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.rename(
        columns={
            'F001N': '基金覆盖家数',
            'F008N': '现金货币类占净资产比例',
            'F007N': '债券固定收益类占净资产比例',
            'F006N': '股票权益类占净资产比例',
            'ENDDATE': '报告期',
            'F005N': '基金市场净资产规模'
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            '报告期',
            '基金覆盖家数',
            '股票权益类占净资产比例',
            '债券固定收益类占净资产比例',
            '现金货币类占净资产比例',
            '基金市场净资产规模',
        ]
    ]
    temp_df["报告期"] = pd.to_datetime(temp_df["报告期"]).dt.date
    temp_df["基金覆盖家数"] = pd.to_numeric(temp_df["基金覆盖家数"])
    temp_df["股票权益类占净资产比例"] = pd.to_numeric(temp_df["股票权益类占净资产比例"])
    temp_df["债券固定收益类占净资产比例"] = pd.to_numeric(temp_df["债券固定收益类占净资产比例"])
    temp_df["现金货币类占净资产比例"] = pd.to_numeric(temp_df["现金货币类占净资产比例"])
    temp_df["基金市场净资产规模"] = pd.to_numeric(temp_df["基金市场净资产规模"])
    return temp_df


if __name__ == "__main__":
    fund_report_stock_cninfo_df = fund_report_stock_cninfo(date="20210630")
    print(fund_report_stock_cninfo_df)

    fund_report_industry_allocation_cninfo_df = fund_report_industry_allocation_cninfo(date="20210930")
    print(fund_report_industry_allocation_cninfo_df)

    fund_report_asset_allocation_cninfo_df = fund_report_asset_allocation_cninfo()
    print(fund_report_asset_allocation_cninfo_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2020/3/17 22:11
Desc: world bank interface
"""
import collections
import datetime
import re
import warnings

import pandas as pd
import tabulate
from decorator import decorator

from . import fetcher

BASE_URL = "https://api.worldbank.org/v2"
COUNTRIES_URL = f"{BASE_URL}/countries"
ILEVEL_URL = f"{BASE_URL}/incomeLevels"
INDICATOR_URL = f"{BASE_URL}/indicators"
LTYPE_URL = f"{BASE_URL}/lendingTypes"
SOURCES_URL = f"{BASE_URL}/sources"
TOPIC_URL = f"{BASE_URL}/topics"
INDIC_ERROR = "Cannot specify more than one of indicator, source, and topic"


class WBSearchResult(list):
    """
    A list that prints out a user-friendly table when printed or returned on the
    command line


    Items are expected to be dict-like and have an "id" key and a "name" or
    "value" key
    """

    def __repr__(self):
        try:
            return tabulate.tabulate(
                [[o["id"], o["name"]] for o in self],
                headers=["id", "name"],
                tablefmt="simple",
            )
        except KeyError:
            return tabulate.tabulate(
                [[o["id"], o["value"]] for o in self],
                headers=["id", "value"],
                tablefmt="simple",
            )


if pd:

    class WBSeries(pd.Series):
        """
        A pandas Series with a last_updated attribute
        """

        _metadata = ["last_updated"]

        @property
        def _constructor(self):
            return WBSeries

    class WBDataFrame(pd.DataFrame):
        """
        A pandas DataFrame with a last_updated attribute
        """

        _metadata = ["last_updated"]

        @property
        def _constructor(self):
            return WBDataFrame


@decorator
def uses_pandas(f, *args, **kwargs):
    """Raise ValueError if pandas is not loaded"""
    if not pd:
        raise ValueError("Pandas must be installed to be used")
    return f(*args, **kwargs)


def parse_value_or_iterable(arg):
    """
    If arg is a single value, return it as a string; if an iterable, return a
    ;-joined string of all values
    """
    if str(arg) == arg:
        return arg
    if type(arg) == int:
        return str(arg)
    return ";".join(arg)


def convert_year_to_datetime(yearstr):
    """return datetime.datetime object from %Y formatted string"""
    return datetime.datetime.strptime(yearstr, "%Y")


def convert_month_to_datetime(monthstr):
    """return datetime.datetime object from %YM%m formatted string"""
    split = monthstr.split("M")
    return datetime.datetime(int(split[0]), int(split[1]), 1)


def convert_quarter_to_datetime(quarterstr):
    """
    return datetime.datetime object from %YQ%# formatted string, where # is
    the desired quarter
    """
    split = quarterstr.split("Q")
    quarter = int(split[1])
    month = quarter * 3 - 2
    return datetime.datetime(int(split[0]), month, 1)


def convert_dates_to_datetime(data):
    """
    Return a datetime.datetime object from a date string as provided by the
    World Bank
    """
    first = data[0]["date"]
    if isinstance(first, datetime.datetime):
        return data
    if "M" in first:
        converter = convert_month_to_datetime
    elif "Q" in first:
        converter = convert_quarter_to_datetime
    else:
        converter = convert_year_to_datetime
    for datum in data:
        datum_date = datum["date"]
        if "MRV" in datum_date:
            continue
        if "-" in datum_date:
            continue
        datum["date"] = converter(datum_date)
    return data


def cast_float(value):
    """
    Return a floated value or none
    """
    try:
        return float(value)
    except (ValueError, TypeError):
        return None


def get_series(
    indicator,
    country="all",
    data_date=None,
    source=None,
    convert_date=False,
    column_name="value",
    keep_levels=False,
    cache=True,
):
    """
    Retrieve indicators for given countries and years

    :indicator: the desired indicator code
    :country: a country code, sequence of country codes, or "all" (default)
    :data_date: the desired date as a datetime object or a 2-tuple with start
        and end dates
    :source: the specific source to retrieve data from (defaults on API to 2,
        World Development Indicators)
    :convert_date: if True, convert date field to a datetime.datetime object.
    :column_name: the desired name for the pandas column
    :keep_levels: if True and pandas is True, don't reduce the number of index
        levels returned if only getting one date or country
    :cache: use the cache
    :returns: WBSeries
    """
    raw_data = get_data(
        indicator=indicator,
        country=country,
        data_date=data_date,
        source=source,
        convert_date=convert_date,
        cache=cache,
    )
    df = pd.DataFrame(
        [[i["country"]["value"], i["date"], i["value"]] for i in raw_data],
        columns=["country", "date", column_name],
    )
    df[column_name] = df[column_name].map(cast_float)
    if not keep_levels and len(df["country"].unique()) == 1:
        df = df.set_index("date")
    elif not keep_levels and len(df["date"].unique()) == 1:
        df = df.set_index("country")
    else:
        df = df.set_index(["country", "date"])
    series = WBSeries(df[column_name])
    series.last_updated = raw_data.last_updated
    return series


def get_data(
    indicator,
    country="all",
    data_date=None,
    source=None,
    convert_date=False,
    pandas=False,
    column_name="value",
    keep_levels=False,
    cache=True,
):
    """
    Retrieve indicators for given countries and years

    :indicator: the desired indicator code
    :country: a country code, sequence of country codes, or "all" (default)
    :data_date: the desired date as a datetime object or a 2-tuple with start
        and end dates
    :source: the specific source to retrieve data from (defaults on API to 2,
        World Development Indicators)
    :convert_date: if True, convert date field to a datetime.datetime object.
    :cache: use the cache
    :returns: list of dictionaries or pandas Series
    """
    if pandas:
        warnings.warn(
            (
                "Argument 'pandas' is deprecated and will be removed in a "
                "future version. Use get_series or get_dataframe instead."
            ),
            PendingDeprecationWarning,
        )
        return get_series(
            indicator=indicator,
            country=country,
            data_date=data_date,
            source=source,
            convert_date=convert_date,
            column_name=column_name,
            keep_levels=keep_levels,
            cache=cache,
        )
    query_url = COUNTRIES_URL
    try:
        c_part = parse_value_or_iterable(country)
    except TypeError:
        raise TypeError("'country' must be a string or iterable'")
    query_url = "/".join((query_url, c_part, "indicators", indicator))
    args = {}
    if data_date:
        if isinstance(data_date, collections.Sequence):
            data_date_str = ":".join((i.strftime("%Y") for i in data_date))
            args["date"] = data_date_str
        else:
            args["date"] = data_date.strftime("%Y")
    if source:
        args["source"] = source
    data = fetcher.fetch(query_url, args, cache=cache)
    if convert_date:
        data = convert_dates_to_datetime(data)
    return data


def id_only_query(query_url, query_id, cache):
    """
    Retrieve information when ids are the only arguments

    :query_url: the base url to use for the query
    :query_id: an id or sequence of ids
    :cache: use the cache
    :returns: WBSearchResult containing dictionary objects describing results
    """
    if query_id:
        query_url = "/".join((query_url, parse_value_or_iterable(query_id)))
    return WBSearchResult(fetcher.fetch(query_url))


def get_source(source_id=None, cache=True):
    """
    Retrieve information on a source

    :source_id: a source id or sequence thereof.  None returns all sources
    :cache: use the cache
    :returns: WBSearchResult containing dictionary objects describing selected
        sources
    """
    return id_only_query(SOURCES_URL, source_id, cache=cache)


def get_incomelevel(level_id=None, cache=True):
    """
    Retrieve information on an income level aggregate

    :level_id: a level id or sequence thereof.  None returns all income level
        aggregates
    :cache: use the cache
    :returns: WBSearchResult containing dictionary objects describing selected
        income level aggregates
    """
    return id_only_query(ILEVEL_URL, level_id, cache=cache)


def get_topic(topic_id=None, cache=True):
    """
    Retrieve information on a topic

    :topic_id: a topic id or sequence thereof.  None returns all topics
    :cache: use the cache
    :returns: WBSearchResult containing dictionary objects describing selected
        topic aggregates
    """
    return id_only_query(TOPIC_URL, topic_id, cache=cache)


def get_lendingtype(type_id=None, cache=True):
    """
    Retrieve information on an income level aggregate

    :level_id: lending type id or sequence thereof.  None returns all lending
        type aggregates
    :cache: use the cache
    :returns: WBSearchResult containing dictionary objects describing selected
        topic aggregates
    """
    return id_only_query(LTYPE_URL, type_id, cache=cache)


def get_country(country_id=None, incomelevel=None, lendingtype=None, cache=True):
    """
    Retrieve information on a country or regional aggregate.  Can specify
    either country_id, or the aggregates, but not both

    :country_id: a country id or sequence thereof. None returns all countries
        and aggregates.
    :incomelevel: desired incomelevel id or ids.
    :lendingtype: desired lendingtype id or ids.
    :cache: use the cache
    :returns: WBSearchResult containing dictionary objects representing each
        country
    """
    if country_id:
        if incomelevel or lendingtype:
            raise ValueError("Can't specify country_id and aggregates")
        return id_only_query(COUNTRIES_URL, country_id, cache=cache)
    args = {}
    if incomelevel:
        args["incomeLevel"] = parse_value_or_iterable(incomelevel)
    if lendingtype:
        args["lendingType"] = parse_value_or_iterable(lendingtype)
    return WBSearchResult(fetcher.fetch(COUNTRIES_URL, args, cache=cache))


def get_indicator(indicator=None, source=None, topic=None, cache=True):
    """
    Retrieve information about an indicator or indicators.  Only one of
    indicator, source, and topic can be specified.  Specifying none of the
    three will return all indicators.

    :indicator: an indicator code or sequence thereof
    :source: a source id or sequence thereof
    :topic: a topic id or sequence thereof
    :cache: use the cache
    :returns: WBSearchResult containing dictionary objects representing
        indicators
    """
    if indicator:
        if source or topic:
            raise ValueError(INDIC_ERROR)
        query_url = "/".join((INDICATOR_URL, parse_value_or_iterable(indicator)))
    elif source:
        if topic:
            raise ValueError(INDIC_ERROR)
        query_url = "/".join(
            (SOURCES_URL, parse_value_or_iterable(source), "indicators")
        )
    elif topic:
        query_url = "/".join((TOPIC_URL, parse_value_or_iterable(topic), "indicators"))
    else:
        query_url = INDICATOR_URL
    return WBSearchResult(fetcher.fetch(query_url, cache=cache))


def search_indicators(query, source=None, topic=None, cache=True):
    """
    Search indicators for a certain regular expression.  Only one of source or
    topic can be specified. In interactive mode, will return None and print ids
    and names unless suppress_printing is True.

    :query: the term to match against indicator names
    :source: if present, id of desired source
    :topic: if present, id of desired topic
    :cache: use the cache
    :returns: WBSearchResult containing dictionary objects representing search
        indicators
    """
    indicators = get_indicator(source=source, topic=topic, cache=cache)
    pattern = re.compile(query, re.IGNORECASE)
    return WBSearchResult(i for i in indicators if pattern.search(i["name"]))


def search_countries(query, incomelevel=None, lendingtype=None, cache=True):
    """
    Search countries by name.  Very simple search.

    :query: the string to match against country names
    :incomelevel: if present, search only the matching incomelevel
    :lendingtype: if present, search only the matching lendingtype
    :cache: use the cache
    :returns: WBSearchResult containing dictionary objects representing
        countries
    """
    countries = get_country(
        incomelevel=incomelevel, lendingtype=lendingtype, cache=cache
    )
    pattern = re.compile(query, re.IGNORECASE)
    return WBSearchResult(i for i in countries if pattern.search(i["name"]))


@uses_pandas
def get_dataframe(
    indicators,
    country="all",
    data_date=None,
    source=None,
    convert_date=False,
    keep_levels=False,
    cache=True,
):
    """
    Convenience function to download a set of indicators and  merge them into a
        pandas DataFrame.  The index will be the same as if calls were made to
        get_data separately.

    :indicators: An dictionary where the keys are desired indicators and the
        values are the desired column names
    :country: a country code, sequence of country codes, or "all" (default)
    :data_date: the desired date as a datetime object or a 2-sequence with
        start and end dates
    :source: the specific source to retrieve data from (defaults on API to 2,
        World Development Indicators)
    :convert_date: if True, convert date field to a datetime.datetime object.
    :keep_levels: if True don't reduce the number of index levels returned if
        only getting one date or country
    :cache: use the cache
    :returns: a pandas DataFrame
    """
    serieses = [
        (
            get_series(
                indicator=indicator,
                country=country,
                data_date=data_date,
                source=source,
                convert_date=convert_date,
                keep_levels=keep_levels,
                cache=cache,
            ).rename(name)
        )
        for indicator, name in indicators.items()
    ]
    print(type(serieses[0]))
    result = None
    for series in serieses:
        if result is None:
            result = series.to_frame()
        else:
            result = result.join(series.to_frame(), how="outer")
    result = WBDataFrame(result)
    result.last_updated = {i.name: i.last_updated for i in serieses}
    return result



# -*- coding:utf-8 -*-
# /usr/bin/env python
"""
Date: 2019/12/9 14:23
Desc: 交易法门-登录函数
"""
from io import BytesIO

from PIL import Image
import requests
from py_mini_racer import py_mini_racer

from mssdk.futures_derivative import jymf_js

from mssdk.futures_derivative.cons import (
    jyfm_init_headers,
    jyfm_login_url,
)


def jyfm_login(account="", password=""):
    """
    交易法门网站登录函数
    :param account: user account
    :type account: str
    :param password: user password
    :type password: str
    :return: headers with cookies
    :rtype: dict
    """
    try:
        pic_url = f"https://www.jiaoyifamen.com/captcha"
        res = requests.get(pic_url)
        f = Image.open(BytesIO(res.content))
        f.show()
        code = input()
        c_func = py_mini_racer.MiniRacer()
        c_func.eval(jymf_js.c.replace(r"\n", ""))
        en_psw = c_func.call("e", password)
        payload = {"nameOrEmail": account, "userPassword": en_psw, "captcha": code}
    except:
        c_func = py_mini_racer.MiniRacer()
        c_func.eval(jymf_js.c.replace(r"\n", ""))
        en_psw = c_func.call("e", password)
        payload = {"nameOrEmail": account, "userPassword": en_psw}
    res = requests.post(jyfm_login_url, json=payload, headers=jyfm_init_headers)
    copy_jyfm_init_headers = jyfm_init_headers.copy()
    copy_jyfm_init_headers["cookie"] = (
        list(dict(res.cookies).keys())[0]
        + "="
        + list(dict(res.cookies).values())[0]
        + "; "
        + list(dict(res.cookies).keys())[1]
        + "="
        + list(dict(res.cookies).values())[1]
    )
    return copy_jyfm_init_headers


if __name__ == "__main__":
    headers = jyfm_login(account="", password="")
    print(headers)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/9/21 16:19
Desc: 巨潮资讯-数据中心-专题统计-股东股本-股东人数及持股集中度
http://webapi.cninfo.com.cn/#/thematicStatistics
"""
import time

import pandas as pd
import requests
from py_mini_racer import py_mini_racer

js_str = """
    function mcode(input) {  
                var keyStr = "ABCDEFGHIJKLMNOP" + "QRSTUVWXYZabcdef" + "ghijklmnopqrstuv"   + "wxyz0123456789+/" + "=";  
                var output = "";  
                var chr1, chr2, chr3 = "";  
                var enc1, enc2, enc3, enc4 = "";  
                var i = 0;  
                do {  
                    chr1 = input.charCodeAt(i++);  
                    chr2 = input.charCodeAt(i++);  
                    chr3 = input.charCodeAt(i++);  
                    enc1 = chr1 >> 2;  
                    enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);  
                    enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);  
                    enc4 = chr3 & 63;  
                    if (isNaN(chr2)) {  
                        enc3 = enc4 = 64;  
                    } else if (isNaN(chr3)) {  
                        enc4 = 64;  
                    }  
                    output = output + keyStr.charAt(enc1) + keyStr.charAt(enc2)  
                            + keyStr.charAt(enc3) + keyStr.charAt(enc4);  
                    chr1 = chr2 = chr3 = "";  
                    enc1 = enc2 = enc3 = enc4 = "";  
                } while (i < input.length);  
          
                return output;  
            }  
"""


def stock_hold_num_cninfo(date: str = "20210630") -> pd.DataFrame:
    """
    巨潮资讯-数据中心-专题统计-股东股本-股东人数及持股集中度
    http://webapi.cninfo.com.cn/#/thematicStatistics
    :param date: choice of {"XXXX0331", "XXXX0630", "XXXX0930", "XXXX1231"}; 从 20170331 开始
    :type date: str
    :return: 股东人数及持股集中度
    :rtype: pandas.DataFrame
    """
    url = "http://webapi.cninfo.com.cn/api/sysapi/p_sysapi1034"
    random_time_str = str(int(time.time()))
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(js_str)
    mcode = js_code.call("mcode", random_time_str)
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "mcode": mcode,
        "Origin": "http://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "http://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    params = {
        "rdate": date,
    }
    r = requests.post(url, headers=headers, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.columns = [
        "本期人均持股数量",
        "股东人数增幅",
        "上期股东人数",
        "本期股东人数",
        "证券简称",
        "证券代码",
        "人均持股数量增幅",
        "变动日期",
        "上期人均持股数量",
    ]
    temp_df = temp_df[
        [
            "证券代码",
            "证券简称",
            "变动日期",
            "本期股东人数",
            "上期股东人数",
            "股东人数增幅",
            "本期人均持股数量",
            "上期人均持股数量",
            "人均持股数量增幅",
        ]
    ]
    temp_df["变动日期"] = pd.to_datetime(temp_df["变动日期"]).dt.date
    temp_df["本期人均持股数量"] = pd.to_numeric(temp_df["本期人均持股数量"])
    temp_df["股东人数增幅"] = pd.to_numeric(temp_df["股东人数增幅"])
    temp_df["上期股东人数"] = pd.to_numeric(temp_df["上期股东人数"])
    temp_df["本期股东人数"] = pd.to_numeric(temp_df["本期股东人数"])
    temp_df["人均持股数量增幅"] = pd.to_numeric(temp_df["人均持股数量增幅"])
    temp_df["上期人均持股数量"] = pd.to_numeric(temp_df["上期人均持股数量"])
    return temp_df


if __name__ == "__main__":
    stock_hold_num_cninfo_df = stock_hold_num_cninfo(date="20210630")
    print(stock_hold_num_cninfo_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/9/11 22:25
Desc: 乐咕乐股网-调查平均持仓数据
https://www.legulegu.com/stockdata/averageposition
"""
import pandas as pd
import requests


def stock_average_position_legu() -> pd.DataFrame:
    """
    乐咕乐股网-调查平均持仓数据
    https://www.legulegu.com/stockdata/averageposition
    :return: 调查平均持仓数据
    :rtype: pandas.DataFrame
    """
    url = "https://www.legulegu.com/stockdata/averageposition/getaverageposition"
    params = {"token": "ac237e85cf6c0a79e2a5299459827f02"}
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json)
    temp_df["date"] = pd.to_datetime(temp_df["date"], unit="ms").dt.date
    del temp_df["id"]
    temp_df.columns = [
        "满仓+融资",
        "80%~99%",
        "60%~79%",
        "40%~59%",
        "20%~39%",
        "0%~19%",
        "平均持仓",
        "上证指数",
        "日期",
    ]
    temp_df = temp_df[
        [
            "日期",
            "上证指数",
            "满仓+融资",
            "80%~99%",
            "60%~79%",
            "40%~59%",
            "20%~39%",
            "0%~19%",
            "平均持仓",
        ]
    ]
    return temp_df


if __name__ == "__main__":
    stock_average_position_legu_df = stock_average_position_legu()
    print(stock_average_position_legu_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/9/4 15:09
Desc: 东方财富网-数据中心-主力数据-基金持仓
http://data.eastmoney.com/zlsj/2020-06-30-1-2.html
"""
from mssdk.utils import demjson
import pandas as pd
import requests


def stock_report_fund_hold(symbol: str = "基金持仓", date: str = "20210331") -> pd.DataFrame:
    """
    东方财富网-数据中心-主力数据-基金持仓
    http://data.eastmoney.com/zlsj/2020-12-31-1-2.html
    :param symbol: choice of {"基金持仓", "QFII持仓", "社保持仓", "券商持仓", "保险持仓", "信托持仓"}
    :type symbol: str
    :param date: 财报发布日期, xxxx-03-31, xxxx-06-30, xxxx-09-30, xxxx-12-31
    :type date: str
    :return: 基金持仓数据
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "基金持仓": "1",
        "QFII持仓": "2",
        "社保持仓": "3",
        "券商持仓": "4",
        "保险持仓": "5",
        "信托持仓": "6",
    }
    date = "-".join([date[:4], date[4:6], date[6:]])
    url = "http://data.eastmoney.com/dataapi/zlsj/list"
    params = {
        "date": date,
        "type": symbol_map[symbol],
        "zjc": "0",
        "sortField": "HOULD_NUM",
        "sortDirec": "1",
        "pageNum": "1",
        "pageSize": "500",
        "p": "1",
        "pageNo": "1",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json['pages']
    big_df = pd.DataFrame()
    for page in range(1, total_page+1):
        params = {
            "date": date,
            "type": symbol_map[symbol],
            "zjc": "0",
            "sortField": "HOULD_NUM",
            "sortDirec": "1",
            "pageNum": page,
            "pageSize": "500",
            "p": page,
            "pageNo": page,
        }
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["data"])
        big_df = big_df.append(temp_df, ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df["index"] = list(range(1, len(big_df)+1))
    big_df.columns = [
        "序号",
        "_",
        "股票简称",
        "_",
        "_",
        "持有基金家数",
        "持股总数",
        "持股市值",
        "_",
        "持股变化",
        "持股变动数值",
        "持股变动比例",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "股票代码",
        "_",
        "_",
    ]
    big_df = big_df[[
        "序号",
        "股票代码",
        "股票简称",
        "持有基金家数",
        "持股总数",
        "持股市值",
        "持股变化",
        "持股变动数值",
        "持股变动比例",
    ]]
    return big_df


def stock_report_fund_hold_detail(symbol: str = "005827", date: str = "20201231") -> pd.DataFrame:
    """
    东方财富网-数据中心-主力数据-基金持仓-明细
    http://data.eastmoney.com/zlsj/ccjj/2020-12-31-008286.html
    :param symbol: 基金代码
    :type symbol: str
    :param date: 财报发布日期, xxxx-03-31, xxxx-06-30, xxxx-09-30, xxxx-12-31
    :type date: str
    :return: 基金持仓-明细数据
    :rtype: pandas.DataFrame
    """
    date = "-".join([date[:4], date[4:6], date[6:]])
    url = "http://datainterface3.eastmoney.com/EM_DataCenter_V3/api/ZLCCMX/GetZLCCMX"
    params = {
        "js": "datatable8848106",
        "tkn": "eastmoney",
        "SHType": "1",
        "SHCode": symbol,
        "SCode": "",
        "ReportDate": date,
        "sortField": "SCode",
        "sortDirec": "1",
        "pageNum": "1",
        "pageSize": "500",
        "cfg": "ZLCCMX",
        "_": "1611579153269",
    }
    r = requests.get(url, params=params)
    data_text = r.text
    data_json = demjson.decode(data_text[data_text.find("{"):-1])
    temp_df = pd.DataFrame(data_json["Data"][0])
    temp_df = temp_df["Data"].str.split("|", expand=True)
    temp_df.reset_index(inplace=True)
    temp_df["index"] = range(1, len(temp_df)+1)
    temp_df.columns = [
        "序号",
        "股票代码",
        "股票简称",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "持股数",
        "持股市值",
        "占总股本比例",
        "占流通股本比例",
    ]
    temp_df = temp_df[[
        "序号",
        "股票代码",
        "股票简称",
        "持股数",
        "持股市值",
        "占总股本比例",
        "占流通股本比例",
    ]]
    return temp_df


if __name__ == "__main__":
    stock_report_fund_hold_df = stock_report_fund_hold(symbol="基金持仓", date="20200630")
    print(stock_report_fund_hold_df)

    stock_report_fund_hold_df = stock_report_fund_hold(symbol="QFII持仓", date="20210331")
    print(stock_report_fund_hold_df)

    stock_report_fund_hold_df = stock_report_fund_hold(symbol="社保持仓", date="20210331")
    print(stock_report_fund_hold_df)

    stock_report_fund_hold_df = stock_report_fund_hold(symbol="券商持仓", date="20210331")
    print(stock_report_fund_hold_df)

    stock_report_fund_hold_df = stock_report_fund_hold(symbol="保险持仓", date="20210331")
    print(stock_report_fund_hold_df)

    stock_report_fund_hold_df = stock_report_fund_hold(symbol="信托持仓", date="20210331")
    print(stock_report_fund_hold_df)

    stock_report_fund_hold_detail_df = stock_report_fund_hold_detail(symbol="005827", date="20210331")
    print(stock_report_fund_hold_detail_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/1/15 16:59
Desc: 个股新闻数据
http://so.eastmoney.com/news/s?keyword=%E4%B8%AD%E5%9B%BD%E4%BA%BA%E5%AF%BF&pageindex=1&searchrange=8192&sortfiled=4
"""
import pandas as pd
import requests


def stock_news_em(stock: str = "601628") -> pd.DataFrame:
    """
    东方财富-个股新闻-最近 20 条新闻
    http://so.eastmoney.com/news/s?keyword=%E4%B8%AD%E5%9B%BD%E4%BA%BA%E5%AF%BF&pageindex=1&searchrange=8192&sortfiled=4
    :param stock: 股票代码
    :type stock: str
    :return: 个股新闻
    :rtype: pandas.DataFrame
    """
    url = "http://searchapi.eastmoney.com//bussiness/Web/GetCMSSearchList"
    params = {
        "type": "8196",
        "pageindex": "1",
        "pagesize": "20",
        "keyword": f"({stock})()",
        "name": "zixun",
        "_": "1608800267874",
    }
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Host": "searchapi.eastmoney.com",
        "Pragma": "no-cache",
        "Referer": "http://so.eastmoney.com/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36",
    }

    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["Data"])
    temp_df.columns = [
        "url",
        "title",
        "_",
        "public_time",
        "content",
    ]
    temp_df['code'] = stock
    temp_df = temp_df[
        [
            "code",
            "title",
            "content",
            "public_time",
            "url",
        ]
    ]
    temp_df["title"] = (
        temp_df["title"].str.replace(r"\(<em>", "", regex=True).str.replace(r"</em>\)", "", regex=True)
    )
    temp_df["content"] = (
        temp_df["content"].str.replace(r"\(<em>", "", regex=True).str.replace(r"</em>\)", "", regex=True)
    )
    temp_df["content"] = (
        temp_df["content"].str.replace(r"<em>", "", regex=True).str.replace(r"</em>", "", regex=True)
    )
    temp_df["content"] = temp_df["content"].str.replace(r"\u3000", "", regex=True)
    temp_df["content"] = temp_df["content"].str.replace(r"\r\n", " ", regex=True)
    return temp_df


if __name__ == "__main__":
    stock_news_em_df = stock_news_em(stock="601318")
    print(stock_news_em_df.info())



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/5/18 15:06
Desc: 增发和配股
东方财富网-数据中心-新股数据-增发-全部增发
http://data.eastmoney.com/other/gkzf.html
东方财富网-数据中心-新股数据-配股
http://data.eastmoney.com/xg/pg/
"""
import requests
import pandas as pd
from mssdk.utils import demjson


def stock_em_qbzf():
    """
    东方财富网-数据中心-新股数据-增发-全部增发
    http://data.eastmoney.com/other/gkzf.html
    :return: 全部增发
    :rtype: pandas.DataFrame
    """
    url = 'http://datainterface.eastmoney.com/EM_DataCenter/JS.aspx'
    params = {
        'st': '5',
        'sr': '-1',
        'ps': '5000',
        'p': '1',
        'type': 'SR',
        'sty': 'ZF',
        'js': '({"pages":(pc),"data":[(x)]})',
        'stat': '0'
    }
    r = requests.get(url, params=params)
    data_text = r.text
    data_json = demjson.decode(data_text[1:-1])
    temp_df = pd.DataFrame([item.split(",") for item in data_json["data"]])
    temp_df.columns = [
        '股票代码',
        '股票简称',
        '发行方式',
        '发行总数',
        '发行价格',
        '最新价',
        '发行日期',
        '增发上市日期',
        '_',
        '增发代码',
        '网上发行',
        '_',
        '_',
        '_',
        '_',
        '_',
        '_',
    ]
    temp_df = temp_df[[
        '股票代码',
        '股票简称',
        '增发代码',
        '发行方式',
        '发行总数',
        '网上发行',
        '发行价格',
        '最新价',
        '发行日期',
        '增发上市日期',
    ]]
    temp_df['锁定期'] = '1-3年'
    return temp_df


def stock_em_pg():
    """
    东方财富网-数据中心-新股数据-配股
    http://data.eastmoney.com/xg/pg/
    :return: 配股
    :rtype: pandas.DataFrame
    """
    url = 'http://datainterface.eastmoney.com/EM_DataCenter/JS.aspx'
    params = {
        'st': '6',
        'sr': '-1',
        'ps': '5000',
        'p': '1',
        'type': 'NS',
        'sty': 'NSA',
        'js': '({data:[(x)],pages:(pc)})',
    }
    r = requests.get(url, params=params)
    data_text = r.text
    data_json = demjson.decode(data_text[1:-1])
    temp_df = pd.DataFrame([item.split(",") for item in data_json["data"]])
    temp_df.columns = [
        '_',
        '_',
        '股票代码',
        '股票简称',
        '配售代码',
        '_',
        '配股比例',
        '配股价',
        '配股前总股本',
        '配股数量',
        '配股后总股本',
        '股权登记日',
        '缴款起始日期',
        '缴款截止日期',
        '上市日',
        '_',
        '_',
        '_',
        '_',
        '_',
        '最新价',
        '_',
    ]
    temp_df = temp_df[[
        '股票代码',
        '股票简称',
        '配售代码',
        '配股数量',
        '配股比例',
        '配股价',
        '最新价',
        '配股前总股本',
        '配股后总股本',
        '股权登记日',
        '缴款起始日期',
        '缴款截止日期',
        '上市日',
    ]]
    temp_df["配股比例"] = '10配' + temp_df["配股比例"]
    return temp_df


if __name__ == '__main__':
    stock_em_qbzf_df = stock_em_qbzf()
    print(stock_em_qbzf_df)

    stock_em_pg_df = stock_em_pg()
    print(stock_em_pg_df)
